@@ -8,6 +8,7 @@ from sqlalchemy import schema as sa_schema
 from . import ops
 from .base import Operations
 from ..util.sqla_compat import _copy
+from ..util.sqla_compat import sqla_2
 
 if TYPE_CHECKING:
     from sqlalchemy.sql.schema import Table
@@ -49,6 +50,11 @@ def alter_column(
             if _count_constraint(constraint):
                 operations.impl.drop_constraint(constraint)
 
+    # some weird pyright quirk here, these have Literal[False]
+    # in their types, not sure why pyright thinks they could be True
+    assert existing_server_default is not True  # type: ignore[comparison-overlap]  # noqa: E501
+    assert comment is not True  # type: ignore[comparison-overlap]
+
     operations.impl.alter_column(
         table_name,
         column_name,
@@ -92,7 +98,11 @@ def drop_column(
 ) -> None:
     column = operation.to_column(operations.migration_context)
     operations.impl.drop_column(
-        operation.table_name, column, schema=operation.schema, **operation.kw
+        operation.table_name,
+        column,
+        schema=operation.schema,
+        if_exists=operation.if_exists,
+        **operation.kw,
     )
 
 
@@ -162,15 +172,35 @@ def add_column(operations: "Operations", operation: "ops.AddColumnOp") -> None:
     column = operation.column
     schema = operation.schema
     kw = operation.kw
+    inline_references = operation.inline_references
+    inline_primary_key = operation.inline_primary_key
 
     if column.table is not None:
         column = _copy(column)
 
     t = operations.schema_obj.table(table_name, column, schema=schema)
-    operations.impl.add_column(table_name, column, schema=schema, **kw)
+    operations.impl.add_column(
+        table_name,
+        column,
+        schema=schema,
+        if_not_exists=operation.if_not_exists,
+        inline_references=inline_references,
+        inline_primary_key=inline_primary_key,
+        **kw,
+    )
 
     for constraint in t.constraints:
         if not isinstance(constraint, sa_schema.PrimaryKeyConstraint):
+            # Skip ForeignKeyConstraint if it was rendered inline
+            # This only happens when inline_references=True AND there's exactly
+            # one FK AND the constraint is single-column
+            if (
+                inline_references
+                and isinstance(constraint, sa_schema.ForeignKeyConstraint)
+                and len(column.foreign_keys) == 1
+                and len(constraint.columns) == 1
+            ):
+                continue
             operations.impl.add_constraint(constraint)
     for index in t.indexes:
         operations.impl.create_index(index)
@@ -197,13 +227,19 @@ def create_constraint(
 def drop_constraint(
     operations: "Operations", operation: "ops.DropConstraintOp"
 ) -> None:
+    kw = {}
+    if operation.if_exists is not None:
+        if not sqla_2:
+            raise NotImplementedError("SQLAlchemy 2.0 required")
+        kw["if_exists"] = operation.if_exists
     operations.impl.drop_constraint(
         operations.schema_obj.generic_constraint(
             operation.constraint_name,
             operation.table_name,
             operation.constraint_type,
             schema=operation.schema,
-        )
+        ),
+        **kw,
     )
 
 
