@@ -31,12 +31,10 @@ from sqlalchemy.sql.elements import BindParameter
 from sqlalchemy.sql.elements import ColumnClause
 from sqlalchemy.sql.elements import TextClause
 from sqlalchemy.sql.elements import UnaryExpression
+from sqlalchemy.sql.naming import _NONE_NAME as _NONE_NAME  # type: ignore[attr-defined] # noqa: E501
 from sqlalchemy.sql.visitors import traverse
 from typing_extensions import TypeGuard
 
-if True:
-    from sqlalchemy.sql.naming import _NONE_NAME as _NONE_NAME  # type: ignore[attr-defined] # noqa: E501
-
 if TYPE_CHECKING:
     from sqlalchemy import ClauseElement
     from sqlalchemy import Identity
@@ -72,6 +70,8 @@ _vers = tuple(
 sqla_14_18 = _vers >= (1, 4, 18)
 sqla_14_26 = _vers >= (1, 4, 26)
 sqla_2 = _vers >= (2,)
+sqla_2_0_25 = _vers >= (2, 25)
+sqla_2_1 = _vers >= (2, 1)
 sqlalchemy_version = __version__
 
 if TYPE_CHECKING:
@@ -81,7 +81,7 @@ if TYPE_CHECKING:
     ) -> Callable[[_CompilerProtocol], _CompilerProtocol]: ...
 
 else:
-    from sqlalchemy.ext.compiler import compiles
+    from sqlalchemy.ext.compiler import compiles  # noqa: I100,I202
 
 
 identity_has_dialect_kwargs = issubclass(schema.Identity, DialectKWArgs)
@@ -293,6 +293,13 @@ else:
         return type_.impl, type_.mapping
 
 
+def _get_table_key(name: str, schema: Optional[str]) -> str:
+    if schema is None:
+        return name
+    else:
+        return schema + "." + name
+
+
 def _fk_spec(constraint: ForeignKeyConstraint) -> Any:
     if TYPE_CHECKING:
         assert constraint.columns is not None
@@ -495,3 +502,9 @@ def is_expression(expr: Any) -> bool:
     if not isinstance(expr, ColumnClause) or expr.is_literal:
         return True
     return False
+
+
+def _inherit_schema_deprecated() -> bool:
+    # at some point in 2.1 inherit_schema was replaced with a property
+    # so that's preset at the class level, while before it wasn't.
+    return sqla_2_1 and hasattr(sqltypes.Enum, "inherit_schema")
